unit Unit1;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, VCLTee.TeEngine, VCLTee.Series,
  Vcl.ExtCtrls, VCLTee.TeeProcs, VCLTee.Chart, Vcl.StdCtrls,
  VCLTee.TeeExtraLegendTool;

type
  TForm1 = class(TForm)
    Chart1: TChart;
    Series1: TBarSeries;
    Series2: TBarSeries;
    Series3: TBarSeries;
    Series4: TBarSeries;
    Series5: TBarSeries;
    Series6: TBarSeries;
    Button1: TButton;
    Series7: TBarSeries;
    procedure Chart1BeforeDrawSeries(Sender: TObject);
  private
    FExtraLegend: TExtraLegendTool;
    FPreTaxColors: array of TColor;
    procedure DrawPercentSymbol(Sender: TObject; Series: TChartSeries; ValueIndex: Integer; aRect: TRect);
    procedure DrawPreTaxRothSymbol(Sender: TObject; Series: TChartSeries;
      ValueIndex: Integer; aRect: TRect);
  public
    { Public declarations }
    constructor Create(aO: TComponent); override;
  end;

var
  Form1: TForm1;

implementation

uses
  VCLTee.TeCanvas;

{$R *.dfm}

{ TForm1 }

procedure GenrateColorArray(const aColor1, aColor2: TColor; var aCOlorArray: array of TColor);
var
  I: Integer;
  lR, lG, lB: word;
  lCount: integer;
  lRDiff, lGDiff, lBDiff: double;
begin

  lR := GetRValue(aColor1);
  lG := GetGValue(aColor1);
  lB := GetBValue(aColor1);

  lCount := Length(aCOlorArray);

  lRDiff := (GetRValue(aColor2) - lR) / lCount;
  lGDiff := (GetGValue(aColor2) - lG) / lCount;
  lBDiff := (GetBValue(aColor2) - lB) / lCount;

  aColorArray[Low(aColorArray)] := aColor1;
  aColorArray[High(aColorArray)] := aColor2;

  for I := (Low(aColorArray) + 1) to (High(aColorArray) - 1) do
    aColorArray[I] := RGB(lR + Round(lRDiff * I), lG + Round(lGDiff * I), lB + Round(lBDiff * I));
end;

function GetColorSawpGB(const aColor: TColor): TColor;
begin
  result := RGB(GetRValue(aColor), GetBValue(aColor), GetGValue(aColor));
end;


constructor TForm1.Create(aO: TComponent);
var
  lColor1: TColor;
  lColor2: TColor;
begin
  inherited;

  lColor1 := $00FFDFBF;
  lColor2 :=  $00885B00;

  SetLength(FPreTaxColors, 3);

  GenrateColorArray(lColor1, lColor2, FPreTaxColors);

  Series1.Color := FPreTaxColors[0];
  Series1.Title := '0.00%';
  Series2.Color := GetColorSawpGB(Series1.Color);
  Series2.ShowInLegend := False;

  Series3.Color := FPreTaxColors[1];
  Series3.Title := '2.00%';
  Series4.Color := GetColorSawpGB(Series3.Color);
  Series4.ShowInLegend := False;

  Series5.Color := FPreTaxColors[2];
  Series5.Title := '4.00%';
  Series6.Color := GetColorSawpGB(Series5.Color);
  Series6.ShowInLegend := False;

  Series1.FillSampleValues(10);
  Series2.FillSampleValues(10);
  Series3.FillSampleValues(10);
  Series4.FillSampleValues(10);
  Series5.FillSampleValues(10);
  Series6.FillSampleValues(10);

  Series7.Active := false;
  Series7.AddNull('Pre-Tax');
  Series7.AddNull('Roth');


  Chart1.Legend.Symbol.OnDraw := DrawPercentSymbol;

  FExtraLegend := TExtraLegendTool.Create(self);
  FExtraLegend.ParentChart := Chart1;
  FExtraLegend.Legend.CustomPosition := True;
  FExtraLegend.Legend.Left := 50;
  FExtraLegend.Legend.Top := 50;
  FExtraLegend.Legend.LegendStyle := lsValues;
  FExtraLegend.Legend.TextStyle := ltsPlain;
  FExtraLegend.Legend.Transparent := True;
  FExtraLegend.Series := Series7;
  FExtraLegend.Active := True;
  FExtraLegend.Legend.Symbol.Width := 45;
  FExtraLegend.Legend.Symbol.OnDraw := DrawPreTaxRothSymbol;
  FExtraLegend.Legend.Title.Text.Text := 'Accounts';

  FExtraLegend.Legend.TextStyle := ltsPlain;
end;

procedure TForm1.Chart1BeforeDrawSeries(Sender: TObject);
begin
  FExtraLegend.Legend.Left := Chart1.Legend.Left;
  FExtraLegend.Legend.Top := Chart1.Legend.Top + Chart1.Legend.Height + 20;
end;

procedure TForm1.DrawPreTaxRothSymbol(Sender: TObject; Series: TChartSeries;
  ValueIndex: Integer; aRect: TRect);
var
  I: Integer;
  lCanvas: TCanvas3D;
  lColor: TColor;
  lLeft: Integer;
  lNumColors: Integer;
  lRight: Integer;
  lWidth: Integer;
  lR: TRect;
begin
  if Length(FPreTaxColors) > 0 then
  begin
    lNumColors := Length(FPreTaxColors);

    lCanvas := Series.ParentChart.Canvas;
    lColor := clBlack;

    lLeft := aRect.Left;
    lWidth := aRect.Width div lNumColors;

    for I := 0 to lNumColors-1 do
    begin
      lColor := FPreTaxColors[I];
      if ValueIndex > 0 then
        lCOlor := GetColorSawpGB(lColor);;

      lCanvas.Pen.Color := lColor;
      lCanvas.Brush.Color := lColor;

      if I = (lNumColors-1) then
        lRight := aRect.Right
      else
        lRight := lLeft + lWidth;

      lR := Rect(lLeft, aRect.Top, lRight, aRect.Bottom);
      lCanvas.FillRect(lR);

      lLeft := lRight;
    end;
  end;
end;

procedure TForm1.DrawPercentSymbol(Sender: TObject; Series: TChartSeries;
  ValueIndex: Integer; aRect: TRect);
var
  lCanvas: TCanvas3D;
  lR: TRect;
  lColor: TColor;
begin
  lCanvas := Series.ParentChart.Canvas;
  lColor := Series.Color;

  lCanvas.Brush.Color := lColor;
  lCanvas.Pen.Color := lColor;

  lR := Rect(aRect.Left, aRect.Top, aRect.Left + aRect.Height, aRect.Top + aRect.Height);
  lCanvas.FillRect(lR);


  lColor := GetColorSawpGB(lColor);

//  lR := Rect(aRect.Left + aRect.Width - aRect.Height, aRect.Top, aRect.Left + aRect.Width, aRect.Top + aRect.Height);
  lR := Rect(aRect.Left + aRect.Width - aRect.Height, aRect.Top, aRect.Right, aRect.Top + aRect.Height);
  lCanvas.Brush.Color := lColor;
  lCanvas.Pen.Color := lColor;
  lCanvas.FillRect(lR);
end;

end.
